type Souvenir = Record
	Title			: String;	//Titulek
	Information		: String;	//Informace o suvenru
	SGuid			: String;	//GUID
	SID				: String;	//ID u novho API cel URL
	SType			: String;	//Typ
	ImageUrl		: String;	//Adresa obrzku - velkho
	ImageThumbUrl	: String;	//Adresa obrzku - malho
	DateDiscovered	: String;	//Kdy zskno
	DateRange		: String;	//Rozsah platnosti
end;

const
	CalcGSAPI_Version = '1.0.3';
	CalcGSAPI_sModuleTmp = '\Stator\Tmp\';

var
	CalcGS_Souvenirs: array of Souvenir;		//Pole suvenr
	
//uvolnovaci funkce modulu
procedure CalcGSAPI_Free;
begin
  Log('Start CalcGSAPI__Free()');
end;
//
//dateparse
function CalcGSAPI_DateParse(Const sd:String):String;
var
	s,y,m,d:String;
begin
	if sd = '' then Result:=sd
	else begin
		s:=sd;
		y:=Fetch(s,'-');
		m:=Fetch(s,'-');
		d:=Fetch(s,'T');
		Result:=d+'.'+m+'.'+y;
	end;
end;
//
//Pevede '' na '0'
function CalcGSAPI_NullToZero(s:String):String;
begin
	if s = '' then Result:='0'
	else Result:=s;
end;
//
function CalcGSAPI_GetSouvenirsData(aParam:TStringList):String;
var
	i: integer;
	Title, SSType, Option, Width: String;
	Isok: boolean;
begin
	Result := ''; SSType := ''; Option := '';Isok:=false;
	if(aParam<>nil) then begin
		For i := 0 to aParam.Count-1 do begin
			//Log('PIDX:'+IntToStr(i));
			if Pos('TITLE',Uppercase(aParam[i])) > 0  then 
				Title := ReplaceString(aParam[i],Copy(aParam[i],1,5)+':','');
			if Pos('TYPE',Uppercase(aParam[i])) > 0  then 
				SSType := ReplaceString(aParam[i],Copy(aParam[i],1,4)+':','');
			if Pos('OPTION',Uppercase(aParam[i])) > 0  then 
				Option := ReplaceString(aParam[i],Copy(aParam[i],1,6)+':','');
			if Pos('WIDTH',Uppercase(aParam[i])) > 0  then 
				Width := ReplaceString(aParam[i],Copy(aParam[i],1,5)+':','');
		end;
		For i := 0 to High(CalcGS_Souvenirs) do begin
			//Log('IDX:'+IntToStr(i));
			if Uppercase(Trim(UtfToAnsi(CalcGS_Souvenirs[i].Title))) = Uppercase(Title) then begin
				Isok:=true;
				break;	
			end;
		end;
		if Isok then begin
			case Uppercase(Option) of
				'URL'	: begin
					if Uppercase(SSType) = 'BIG' then
						Result := CalcGS_Souvenirs[i].ImageUrl
					else
						Result := CalcGS_Souvenirs[i].ImageThumbUrl;
				end;
				'INFO'	:Result := UtfToAnsi(CalcGS_Souvenirs[i].Information);
				else begin
					if Uppercase(SSType) = 'BIG' then
						Result := '<a href="https://www.geocaching.com/souvenir/?guid='+CalcGS_Souvenirs[i].SGuid+'" target="_blank"><img src="'+CalcGS_Souvenirs[i].ImageUrl + '" width="' + ReplaceString(Width,'%','') +'" title="'+UtfToAnsi(CalcGS_Souvenirs[i].Title)+' ('+CalcGS_Souvenirs[i].DateDiscovered+')" style="max-height:480px;max-width:480px;" /></a>'
					else
						Result := '<a href="https://www.geocaching.com/souvenir/?guid='+CalcGS_Souvenirs[i].SGuid+'" target="_blank"><img src="'+CalcGS_Souvenirs[i].ImageThumbUrl + '" width="' + ReplaceString(Width,'%','') +'" title="'+UtfToAnsi(CalcGS_Souvenirs[i].Title)+' ('+CalcGS_Souvenirs[i].DateDiscovered+')" style="max-height:100px;max-width:100px;" /></a>';
				end;
			end;
		end;
	end;
end;

procedure SaveTmpFile(s,f:String);
begin
	if FileExists(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f) then begin
		if DeleteFile(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f) then begin
			StringToFile(s,GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		end;
	end
	else begin 
		StringToFile(s,GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
	end;
end;

//hlavni funkce modulu
function CalcGSAPI_Run(aPar:TStringList):string;
var
	sxmlData, sxmlRes, s, sa, sRef, f, msg: String;
	i,j,k,n,m,mf,nx,idx,nPerPage: integer;
	dt: TDateTime;
	sJson, sj, sjj, sUserName, sReq:string;
  rJson:TSimpleJSon;
	iErr,nLogs:integer;
	aList:TStringList;
begin
	Result:='';
	//----------------------------------------------------------------------------------------------
	//registrovat fce
	GlobalVarRegisterReplaceFunction('CalcGSAPI_GetSouvenirsData',@CalcGSAPI_GetSouvenirsData);
	//----------------------------------------------------------------------------------------------
	Log('Start CalcGSAPI_Run()');
	//
	sUserName:=GEOGET_OWNER;
	//TMP/API	
	if not (DirectoryExists(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp)) then begin
		if not (ForceDirectories(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp)) then begin
			msg :='ERR: '+_('Could not create directory: ') + GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp;
			AddError(msg);
			exit;
		end;
	end;
	//USER JSON
	//zpracovn dat uivatele
	{$ifdef DEBUG_HELPER} LDH('API User: zpracovn'); {$endif}
	sReq:='v1/users?usernames='+encodeurlelement(AnsiToUtf(sUserName))
		+'&fields=username,referenceCode,findCount,hideCount,favoritePoints,membershipLevelId,'
		+'avatarUrl,bannerUrl,url,homeCoordinates,trackableFindCounts,trackableOwnedCounts,'
		+'geocacheFindCounts,geocacheHideCounts,'
		+'geocacheLimits';
	//file
	f:='GetYourUserProfile'+GlobalVarGet('Profile',false)+'.txt';
	mf:=0; //zapsat do TMP
	//
	if FileExists(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f) then begin
		dt := GetFileTime(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		s := FileToString(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		//
		Log('Delay user data: '+IntToStr(4-round(double(Now - dt) * 24))+' hour');
		if round(double(Now - dt) * 24) < 4 then begin	//4 hodiny delay ne se natou nov data pro API
			mf := 1;                                      //zruit zpis do TMP
			sJson := s;                                   //pouiju data ze zlohy (TEMP)
			Log('Data from temp file: '+ GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
			iErr := 200;																	//Nen chyba - zpracovat
		end
		else begin        
			iErr:=GcLiveRest('GET',sReq,'','',sJson);        //Z API
		end;
	end
	else begin
		iErr:=GcLiveRest('GET',sReq,'','',sJson);                //Z API
	end;
{$ifdef DEBUG_HELPER} LDH('ERR: '+IntToStr(iErr)); {$endif}
  if(iErr<>200) then begin
    ShowMessage('ERR: Chyba pri zjistovani informaci o uzivateli');
    exit;
  end;
  if mf = 0 then sJson:='{root:'+sJson+'}';  //m=0 nov - prava na legln JSON, m <> 0 ze zlohy
  rJson:=TSimpleJson.Create();
  try
    if(rJson.ParseString(sJson)) then begin
			sRef:=rJson.AsString('root[0].referenceCode');
			GlobalVarSet('CalcGS_UserRef',sRef);
			GlobalVarSet('CalcGS_UserID',IntToStr(GcApiCode2Num(s)));
			//Rzn uivatelsk...
			GlobalVarSet('CalcGS_UserAvatarUrl',rJson.AsString('root[0].avatarUrl'));
			GlobalVarSet('CalcGS_UserBannerUrl',rJson.AsString('root[0].bannerUrl'));
			GlobalVarSet('CalcGS_UserUrl',rJson.AsString('root[0].url'));
			GlobalVarSet('CalcGS_UserFindCount',CalcGSAPI_NullToZero(rJson.AsString('root[0].findCount')));
			GlobalVarSet('CalcGS_GeocachesFindCount',CalcGSAPI_NullToZero(rJson.AsString('root[0].findCount')));
			GlobalVarSet('CalcGS_UserHideCount',CalcGSAPI_NullToZero(rJson.AsString('root[0].hideCount')));
			GlobalVarSet('CalcGS_GeocachesHideCount',CalcGSAPI_NullToZero(rJson.AsString('root[0].hideCount')));
			GlobalVarSet('CalcGS_UserHomeCoordinates',rJson.AsString('root[0].homeCoordinates.latitude')+'|'+rJson.AsString('root[0].homeCoordinates.longitude'))
			GlobalVarSet('CalcGS_UserMemberType',GcApiUserType2Text(rJson.AsInteger('root[0].membershipLevelId')));
			GlobalVarSet('CalcGS_UserName',rJson.AsString('root[0].username'));
			GlobalVarSet('CalcGS_UserPublicGuid','-');																			//U neexistuje
			//FP
			GlobalVarSet('CalcGS_FPPointsAvailable',CalcGSAPI_NullToZero(rJson.AsString('root[0].favoritePoints')));		
			//Find-----------------------------------------------------------------------------------
			j:= rJson.AsArraySize('root[0].geocacheFindCounts'); n := 0;
			For i:=0 to j-1 do begin
				n := n + rJson.AsInteger('root[0].geocacheFindCounts['+IntToStr(i)+'].count');
			end;
			GlobalVarSet('CalcGS_GeocachesFindCountByType',IntToStr(n));
			//
			j:= rJson.AsArraySize('root[0].geocacheHideCounts'); n := 0;
			For i:=0 to j-1 do begin
				n := n + rJson.AsInteger('root[0].geocacheHideCounts['+IntToStr(i)+'].count');
			end;
			GlobalVarSet('CalcGS_GeocachesHideCountByType',IntToStr(n));
			//---------------------------------------------------------------------------------------
			//GC/TB----------------------------------------------------------------------------------
			j := rJson.AsArraySize('root[0].trackableFindCounts'); n := 0;k := 0;
			GlobalVarSet('CalcGS_TrackablesFindTypeCount', IntToStr(j));
			For i:=0 to j-1 do begin
				m := rJson.AsInteger('root[0].trackableFindCounts['+IntToStr(i)+'].count');
				n := n + m;
				//
				sa := rJson.AsString('root[0].trackableFindCounts['+IntToStr(i)+'].trackableType.name');
				if sa = 'Travel Bug Dog Tags' then begin
					k := k + m;
				end;
			end;
			GlobalVarSet('CalcGS_TrackablesFoundCountByType',IntToStr(n));
			GlobalVarSet('CalcGS_TrackablesFoundCount',IntToStr(n));
			GlobalVarSet('CalcGS_TrackablesTBCount',IntToStr(k));
			GlobalVarSet('CalcGS_TrackablesGCCount',IntToStr(n-k));
			//
			j:= rJson.AsArraySize('root[0].trackableOwnedCounts'); n := 0;
			GlobalVarSet('CalcGS_TrackablesOwnedTypeCount', IntToStr(j));
			For i:=0 to j-1 do begin
				n := n + rJson.AsInteger('root[0].trackableOwnedCounts['+IntToStr(i)+'].count');
			end;
			GlobalVarSet('CalcGS_TrackablesOwnedCountByType',IntToStr(n));
			GlobalVarSet('CalcGS_TrackablesOwnedCount',IntToStr(n));
			//---------------------------------------------------------------------------------------
    end;
  finally
    rJson.Free();
  end;
	//Save
	if mf = 0 then SaveTmpFile(sJson,f);
	
	//SOUVENIRS JSON
	//Zpracovn seznamu "suvenr"
	{$ifdef DEBUG_HELPER} LDH('API Souvenirs: zpracovn'); {$endif}
	idx:=0;						//Index 0
	nx:=1;						//Strnka 1
	nPerPage:=50;			//Poet ke na strnku 50 (max)
	//file
	f:='GetYourSouvenirs'+GlobalVarGet('Profile',false)+'.txt';
	mf:=0; //zapsat do TMP
	//
	if FileExists(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f) then begin
		dt := GetFileTime(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		s := FileToString(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		//
		Log('Delay souvenirs data: '+IntToStr(4-round(double(Now - dt) * 24))+' hour');
		if round(double(Now - dt) * 24) < 4 then begin	//12 hodiny delay ne se natou nov data pro API
			mf := 1;                                      //zruit zpis do TMP
			sJson := s;                                   //pouiju data ze zlohy (TEMP)
			Log('Data from temp file: '+ GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
			iErr := 200;																	//Nen chyba - zpracovat
		end;
	end;
	if mf=0 then begin
		sJson := '';
		while nx>0 do begin
			sReq:='/v1/users/'+sRef+'/souvenirs?fields='
				+ 'title,description,imagePath,thumbImagePath,foundDateUtc,url'
				+ '&take='+IntToStr(nPerPage)+'&skip='+IntToStr(idx);
			//		
			iErr:=GcLiveRest('GET',sReq,'','',sj);                //Z API
			{$ifdef DEBUG_HELPER} LDH('ERR: '+IntToStr(iErr)); {$endif}
			if(iErr<>200) then begin
				ShowMessage('ERR: Chyba pri zjistovani informaci o uzivateli');
				exit;
			end;
			sjj :='{root:'+sj+'}';
			rJson:=TSimpleJson.Create();
			try
				if(rJson.ParseString(sjj)) then begin
					nLogs:=rJson.AsArraySize('root');
				end;
			finally
				rJson.Free();
			end;
			if nx > 1 then sj := ReplaceString(sj,'[{','{');
			if nLogs = nPerPage then sj := ReplaceString(sj,'}]','},');
			//
			sJson := sJson+sj;
			//
			if nLogs < nPerPage then begin
				nx := 0;	//konec zvonec
			end
			else begin
				Inc(nx);	//Poet natench strnek
				idx:=idx+nPerPage;
			end;
		end;
		sJson:='{root:'+sJson+'}';  //m=0 nov - prava na legln JSON, m <> 0 ze zlohy
	end;
	//
  rJson:=TSimpleJson.Create();
	aList:=TStringList.Create(); aList.Sorted:=true;
  try
		if(rJson.ParseString(sJson)) then begin
			n := rJson.AsArraySize('root');
			{$ifdef DEBUG_HELPER} LDH('Elements: ' +IntToStr(n)); {$endif}
			GlobalVarSet('CalcGS_SouvenirsCount',IntToStr(n));
			if n > 0 then begin
				//
				SetLength(CalcGS_Souvenirs, n);				//nastavi pole suvenr
				//
				for i:=0 to n-1 do begin
					s := rJson.AsString('root['+IntToStr(i)+'].title');
					CalcGS_Souvenirs[i].Title:=s; aList.Add(UtfToAnsi(s));
					CalcGS_Souvenirs[i].Information:=rJson.AsString('root['+IntToStr(i)+'].description');
					s := rJson.AsString('root['+IntToStr(i)+'].url');
					sj := s;
					sa := Fetch(s,'guid=');
					CalcGS_Souvenirs[i].SGuid:=s;	//GUID
					CalcGS_Souvenirs[i].SID:=sj; //Cel URL na Souvenir
					CalcGS_Souvenirs[i].SType:='Null';
					CalcGS_Souvenirs[i].ImageUrl:=rJson.AsString('root['+IntToStr(i)+'].imagePath');
					CalcGS_Souvenirs[i].ImageThumbUrl:=rJson.AsString('root['+IntToStr(i)+'].thumbImagePath');
					CalcGS_Souvenirs[i].DateDiscovered:=CalcGSAPI_DateParse(rJson.AsString('root['+IntToStr(i)+'].foundDateUtc'));
					CalcGS_Souvenirs[i].DateRange:='Null';
				end;
				GlobalVarSet('CalcGS_Souvenirs',ReplaceString(Trim(aList.Text),CRLF,' | '));
			end;
		end;
	finally
    rJson.Free();
		aList.Free();
  end;
	//Save
	if mf = 0 then SaveTmpFile(sJson,f);
	
	//IMAGES JSON
	//Zpracovn seznamu "obrzk"
	{$ifdef DEBUG_HELPER} LDH('API Images: zpracovn'); {$endif}
	idx:=0;						//Index 0
	nx:=1;						//Strnka 1
	nPerPage:=50;			//Poet ke na strnku 50 (max)
	//file
	f:='GetYourImages'+GlobalVarGet('Profile',false)+'.txt';
	mf:=0; //zapsat do TMP
	//
	if FileExists(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f) then begin
		dt := GetFileTime(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		s := FileToString(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		//
		Log('Delay images data: '+IntToStr(24-round(double(Now - dt) * 24))+' hour');
		if round(double(Now - dt) * 24) < 24 then begin	//24 hodiny delay ne se natou nov data pro API
			mf := 1;                                      //zruit zpis do TMP
			sJson := s;                                   //pouiju data ze zlohy (TEMP)
			Log('Data from temp file: '+ GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
			iErr := 200;																	//Nen chyba - zpracovat
		end;
	end;
	if mf=0 then begin
		sJson := '';
		while nx>0 do begin
			sReq:='/v1/users/'+sRef+'/images?fields='
				+ 'referenceCode'
				+ '&take='+IntToStr(nPerPage)+'&skip='+IntToStr(idx);
			//		
			iErr:=GcLiveRest('GET',sReq,'','',sj);                //Z API
			{$ifdef DEBUG_HELPER} LDH('ERR: '+IntToStr(iErr)); {$endif}
			if(iErr<>200) then begin
				ShowMessage('ERR: Chyba pri zjistovani informaci o uzivateli');
				exit;
			end;
			sjj :='{root:'+sj+'}';
			rJson:=TSimpleJson.Create();
			try
				if(rJson.ParseString(sjj)) then begin
					nLogs:=rJson.AsArraySize('root');
				end;
			finally
				rJson.Free();
			end;
			if nx > 1 then sj := ReplaceString(sj,'[{','{');
			if nLogs = nPerPage then sj := ReplaceString(sj,'}]','},');
			//
			sJson := sJson+sj;
			//
			if nLogs < nPerPage then begin
				nx := 0;	//konec zvonec
			end
			else begin
				Inc(nx);	//Poet natench strnek
				idx:=idx+nPerPage;
			end;
		end;
		sJson:='{root:'+sJson+'}';  //m=0 nov - prava na legln JSON, m <> 0 ze zlohy
	end;
	//
  rJson:=TSimpleJson.Create();
  try
		if(rJson.ParseString(sJson)) then begin
			n := rJson.AsArraySize('root');
			{$ifdef DEBUG_HELPER} LDH('Elements: ' +IntToStr(n)); {$endif}
			GlobalVarSet('CalcGS_UserGalleryImagesCount',IntToStr(n));
		end;
	finally
    rJson.Free();
  end;
	//Save
	if mf = 0 then SaveTmpFile(sJson,f);
	
	//Countries JSON
	{$ifdef DEBUG_HELPER} LDH('API Countries: zpracovn'); {$endif}
	f:='GetCountries'+GlobalVarGet('Profile',false)+'.txt';
	mf:=0; //zapsat do TMP
	//
	if FileExists(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f) then begin
		dt := GetFileTime(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		s := FileToString(GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
		//
		Log('Delay country data: '+IntToStr(168-round(double(Now - dt) * 24))+' hour');
		if round(double(Now - dt) * 24) < 168 then begin	//tden delay ne se natou nov data pro API
			mf := 1;                                      //zruit zpis do TMP
			sJson := s;                                   //pouiju data ze zlohy (TEMP)
			Log('Data from temp file: '+ GEOGET_SCRIPTDIR + CalcGSAPI_sModuleTmp + f);
			iErr := 200;																	//Nen chyba - zpracovat
		end;
	end;
	if mf=0 then begin
		sJson := '';
		sReq:='/v1/countries'
		//		
		iErr:=GcLiveRest('GET',sReq,'','',sJson);      //Z API
		sJson:='{root:'+sJson+'}';  									 //prava na legln JSON, m <> 0 ze zlohy
	end;
	//zpracovn
	rJson:=TSimpleJson.Create();
  try
		if(rJson.ParseString(sJson)) then begin
			n := rJson.AsArraySize('root');
			{$ifdef DEBUG_HELPER} LDH('Elements: ' +IntToStr(n)); {$endif}
			GlobalVarSet('CalcGS_StatActiveCountries',IntToStr(n));
		end;
	finally
    rJson.Free();
  end;
	//Save
	if mf = 0 then SaveTmpFile(sJson,f);
	//
	Log('End of CalcGSAPI_Run()');
end;