const
  MyOwnLife_GraphHeight=280;                             // vka grafu nlezu
  MyOwnLife_GraphEventHeight=120;                        // vka eventovho grafu
  MyOwnLife_GraphWidth=350;                              // ka grafu

type MyOwnLife_TCacheData = Record //---------------------------------------------
  sID        : String;      //ID cache - GC....
  dCreate    : TDateTime;   //datum vytvoreni
  dArchive   : TDateTime;   //datum archivace (-1 = neni do ted)
  sGuid      : String;      //guid
  sType      : String;      //typ kese
  bEvent     : Boolean;     //priznak, zda jde o event
  sName      : String;      //jmeno kese
  sOwnerID   : String;      //ID autora kese
  bArchived  : Boolean;     //archivovana  (neni to duplikovani? dArchive a iStatus to obsahuji taky)
  iStatus    : Integer;     //stav kese
  sDifficult : String;      //difficulty
  sTerrain   : String;      //terrain
  sSize      : String;      //size
  sElevation : String;      //elevation
  sFoundIt   : String;      //found it / attended
  //dopocitane ------------------------------------------------------
  dPublic    : TDateTime;   //datum publikacniho logu
  dLastFound : TDateTime;   //datum posledniho nalezu
  nExistDays : Integer;     //delka existence cache ve dnech
  nLogs      : Integer;     //pocet logu celkem
  nFoundIt   : Integer;     //pocet nalezu
  nDNF       : Integer;     //pocet nenalezenych
  nWillAttend: Integer;     //pocet prihlasek na event
  sFTF       : String;      //FTFkari
  sSTF       : String;      //STFkari
  sTTF       : String;      //TTFkari
  //Favorite points--------------------------------------------------
  nFP        : integer;     //pocet FP
  //GCVote-----------------------------------------------------------
  nGcVoteCount : integer;
  fGcVoteMedian: Extended;
  //logy-------------------------------------------------------------
  nWordsTotal: integer;     //celkovy pocet slov na vsech nalezovych lozich
                            //(a vsech nasledujicich WN logach pro pripad, ze
                            //se autor nevesel do dane delky textu)
  nWordsMin,nWordsMax: integer;  //nejkratsi a nejdelsi log
  sWordsMinLID,sWordsMaxLID: string; //odkaz na logy
  sWordsMinFinder,sWordsMaxFinder:string;  //a jejich autori
end; //--------------------------------------------------------------
  
var
  MyOwnLife_Param: record
      sCaches:string;     	// seznam GC kodu kesi, ktere statistika ma zpracovavat
      sCachesSQL:string;  	// totez, ale v SQL formatu pro frazi IN
      sTagXTF:string;     	// tag pro ziskani xTF nalezcu
      bWordLogs:boolean;  	// pocitat slova v logach
      bFP:boolean;        	// informace o Favorit points
  end;
  MyOwnLife_aCacheData:array of MyOwnLife_TCacheData;
  MyOwnLife_dGraphFrom:TDateTime;	// globalni minimum data zalozeni pro skalovani osy grafu
  MyOwnLife_nCaches:integer;
  MyOwnLife_iMax:integer;	// globalni maximum nalezu pro skalovani osy grafu (jen kese, ktere nejsou Eventy)

function MyOwnLife_GetIdElement(sMainID:string; idx:integer):string;
begin
  Result:=sMainID+'_'+MyOwnLife_aCacheData[idx].sID;
end;

function MyOwnLife_IsChar(ch:char):boolean;
begin
  Result:=( ((ch>='a') AND (ch<='z'))
           OR ((ch>='A') AND (ch<='Z'))
           OR ((ch>='0') AND (ch<='9'))
	         OR (ch='_') OR (ch='[') OR (ch=']') OR (ch='(') OR (ch=')')
           OR (ch=':') OR (ch=';') OR (ch='!') OR (ch='?') OR (ch='/')
           OR (ch='{') OR (ch='}') OR (ch='|')  OR (ch='#')OR (ch='^'));
end;
  
function MyOwnLife_CountWords(str:string):integer; //spocita pocet slov textu
var  i: integer;
     bChar:boolean;
begin
  Result:=0;
  bChar:=FALSE;
  for i:=1 to Length(str) do begin
    if(MyOwnLife_IsChar(str[i])) then begin
      if(not bChar) then begin
        Inc(Result);
        bChar:=TRUE;
      end;
    end
    else bChar:=FALSE;
  end;
end;  

function MyOwnLife_PrepareData(): integer; //vraci pocet nactenych kesi
var
  sSql:string;
  tab:TSQLiteTable;
begin
  Result:=0;
  StatorBusy(_('Preparing owned caches ...'));
  Log(_('Processing owned caches: ')+MyOwnLife_Param.sCaches);
  if(MyOwnLife_Param.sCaches='') then exit; //neni co zpracovavat
  MyOwnLife_dGraphFrom:=Now();
  sSql:='SELECT gc.id '                                             //%0
        //+'replace(replace(gc.name,'''',''>''),"''","") ' //nevim, proc tam mel Drak tuto nahradu
        +', name, gc.guid, gc.gs_ownerid, gc.cachetype, gc.dthidden'+CRLF //%1-%5
        +', gc.cachestatus'+CRLF                                           //%6
        +', CASE WHEN cachestatus=2 THEN '+CRLF                            //%7
        +  '(SELECT CAST(gl.dt AS integer) FROM geolog gl '+CRLF
        +    'WHERE gl.id=gc.id and gl.type=''Archive'' '+CRLF
        +  'ORDER BY CAST(gl.dt AS integer) DESC LIMIT 1)'+CRLF
        +' ELSE -1'+CRLF
        +' END enddatetime'+CRLF
        +', gc.difficulty, gc.terrain, gc.cachesize'+CRLF                 //%8-%10
        +', IFNULL((SELECT value FROM geotagvalue WHERE key=gt.ptrvalue),0) elevation'+CRLF //%11
        +', MIN(CASE WHEN ifnull(gl.dt,'''') = '''' '+CRLF              //%12
        +'   THEN (select glx.dt from geolog glx where gc.id = glx.id and glx.type = ''Found it'' order by glx.dt ASC LIMIT 1)'+CRLF
        +'   ELSE gl.dt'+CRLF
        +' END) dtpublish'+CRLF
        +', (SELECT count(*) FROM geolog glx WHERE glx.id = gc.id AND '+CRLF //%13
        +'   (glx.type=''Found it'' OR glx.type=''Attended'' OR glx.type=''Webcam Photo Taken'')) FoundIt'+CRLF
        +', (SELECT count(*) FROM geolog glx WHERE glx.id = gc.id AND '+CRLF //%14
        +'   (glx.type=''Didn''''t find it'')) DNF'+CRLF
        +', IFNULL((SELECT value FROM geotagvalue WHERE key=gtFav.ptrvalue),0) fp'+CRLF //%15
        +', IFNULL((SELECT value FROM geotagvalue WHERE key=gtFTF.ptrvalue),'''') FTF'+CRLF //%16
        +', IFNULL((SELECT value FROM geotagvalue WHERE key=gtSTF.ptrvalue),'''') STF'+CRLF //%17
        +', IFNULL((SELECT value FROM geotagvalue WHERE key=gtTTF.ptrvalue),'''') TTF'+CRLF //%18
        +', IFNULL((SELECT replace(value,''x'','''') FROM geotagvalue WHERE key=gt1.ptrvalue),0) GcVoteCount'+CRLF //%19
        +', IFNULL((SELECT value FROM geotagvalue WHERE key=gt2.ptrvalue), 0) GcVoteMedian'+CRLF    //%20
{       // Hodnoceni na geocaching.cz je zruseno, nema smysl jej tu zpracovavat
        //+', IFNULL((SELECT value FROM geotagvalue WHERE key=gt3.ptrvalue),''0x'') CZCount'+CRLF  //%21
        //+', IFNULL((SELECT value FROM geotagvalue WHERE key=gt4.ptrvalue), 0) CZValue '+CRLF;     //%22
}
        +' FROM geocache gc'+CRLF
        //pokud bude vice publikacnich logu, bude keska pridana do seznamu vicekrat!!!
        //zatim neni osetreno
        +' LEFT JOIN geolog gl ON (gc.id=gl.id and gl.type=''Publish Listing'')'+CRLF
        +' LEFT JOIN geotag gt ON (gc.id=gt.id and gt.ptrkat IN (SELECT key FROM geotagcategory WHERE value=''Elevation'' COLLATE NOCASE))'+CRLF
        +' LEFT JOIN geotag gtFav ON (gc.id=gtFav.id and gtFav.ptrkat IN (SELECT key FROM geotagcategory WHERE value=''favorites'' COLLATE NOCASE))'+CRLF
        +' LEFT JOIN geotag gt1 ON (gc.id=gt1.id and gt1.ptrkat IN (SELECT key FROM geotagcategory WHERE value=''GCvote-Count'' COLLATE NOCASE))'+CRLF
        +' LEFT JOIN geotag gt2 ON (gc.id=gt2.id and gt2.ptrkat IN (SELECT key FROM geotagcategory WHERE value=''GCvote-median'' COLLATE NOCASE))'+CRLF
{        // Hodnoceni na geocaching.cz je zruseno, nema smysl jej tu zpracovavat
        //+' LEFT JOIN geotag gt3 ON (gc.id=gt3.id and gt3.ptrkat IN (SELECT key FROM geotagcategory WHERE value=''Hodnoceni-Pocet'' COLLATE NOCASE))'
        //+' LEFT JOIN geotag gt4 ON (gc.id=gt4.id and gt4.ptrkat IN (SELECT key FROM geotagcategory WHERE value=''Hodnoceni'' COLLATE NOCASE))'
}
//sem jeste pridame tabulku pro FTF, STF a TTF
        +' LEFT JOIN geotag gtFTF ON (gc.id=gtFTF.id and gtFTF.ptrkat IN (SELECT key FROM geotagcategory WHERE value='''+ReplaceString(MyOwnLife_Param.sTagXTF,'???','FTF')+''' COLLATE NOCASE))'+CRLF
        +' LEFT JOIN geotag gtSTF ON (gc.id=gtSTF.id and gtSTF.ptrkat IN (SELECT key FROM geotagcategory WHERE value='''+ReplaceString(MyOwnLife_Param.sTagXTF,'???','STF')+''' COLLATE NOCASE))'+CRLF
        +' LEFT JOIN geotag gtTTF ON (gc.id=gtTTF.id and gtTTF.ptrkat IN (SELECT key FROM geotagcategory WHERE value='''+ReplaceString(MyOwnLife_Param.sTagXTF,'???','TTF')+''' COLLATE NOCASE))'+CRLF
        +' WHERE gc.id IN ('+MyOwnLife_Param.sCachesSQL+')'+CRLF
//        +' WHERE gc.id IN ('+GlobalVarGet('OwnedIDsForInPhrase',false)+')'
        +' GROUP BY gc.id'+CRLF
        +' ORDER BY dtpublish ASC'+CRLF
        ;
  //Log('WWWWW SQL='+sSql);
  tab:=GEOGET_DB.GetTable(sSql, false);
  try
    while (not tab.Eof) do begin
      SetLength(MyOwnLife_aCacheData,Result+1);
      MyOwnLife_aCacheData[Result].nWordsTotal:=0;
      MyOwnLife_aCacheData[Result].nWordsMin:=999999;
      MyOwnLife_aCacheData[Result].nWordsMax:=0;
      MyOwnLife_aCacheData[Result].sWordsMinLID:='';
      MyOwnLife_aCacheData[Result].sWordsMaxLID:='';
      MyOwnLife_aCacheData[Result].sWordsMinFinder:='';
      MyOwnLife_aCacheData[Result].sWordsMinFinder:='';
      MyOwnLife_aCacheData[Result].sId:=tab.FieldAsString(0);
      MyOwnLife_aCacheData[Result].sName:=Trim(UtfToAnsi(tab.FieldAsString(1)));
      //Log('sId='+MyOwnLife_aCacheData[Result].sId+', sName='+MyOwnLife_aCacheData[Result].sName);
      MyOwnLife_aCacheData[Result].sGuid:=tab.FieldAsString(2);
      MyOwnLife_aCacheData[Result].sOwnerID:=tab.FieldAsString(3);
      //Log('sGuid='+MyOwnLife_aCacheData[Result].sGuid+', sOwnerID='+MyOwnLife_aCacheData[Result].sOwnerID);
      MyOwnLife_aCacheData[Result].sType:=tab.FieldAsString(4);
      case MyOwnLife_aCacheData[Result].sType of
        'Event Cache',
        'Cache In Trash Out Event',
        'Mega-Event Cache',
        'Giga-Event Cache',
        'Lost and Found Event Cache': MyOwnLife_aCacheData[Result].bEvent:=true;
        else MyOwnLife_aCacheData[Result].bEvent:=false;
      end;
      MyOwnLife_aCacheData[Result].dCreate:=IntToDate(tab.FieldAsString(5));
      //do meritka osy X nepocitame eventy, protoze ty nemaji grafy
      if(MyOwnLife_dGraphFrom>MyOwnLife_aCacheData[Result].dCreate) and (not MyOwnLife_aCacheData[Result].bEvent)
        then MyOwnLife_dGraphFrom:=MyOwnLife_aCacheData[Result].dCreate;
      //Log('sType='+MyOwnLife_aCacheData[Result].sType+', dCreate='+DateToStr(MyOwnLife_aCacheData[Result].dCreate));
      MyOwnLife_aCacheData[Result].iStatus:=tab.FieldAsInteger(6);
      if(tab.FieldAsInteger(7)<0) then begin  //pokud neni archivacni log, povazujeme ji za enabled
        MyOwnLife_aCacheData[Result].dArchive:=Now();
        MyOwnLife_aCacheData[Result].bArchived:=false;
      end
      else begin
        MyOwnLife_aCacheData[Result].dArchive:=IntToDate(tab.FieldAsString(7));
        MyOwnLife_aCacheData[Result].bArchived:=true;
      end;
      //Log('iStatus='+IntToStr(MyOwnLife_aCacheData[Result].iStatus)+', dArchive='+DateToStr(MyOwnLife_aCacheData[Result].dArchive)
      //  +', bArchived='+IntToStr(MyOwnLife_aCacheData[Result].bArchived as integer));
      MyOwnLife_aCacheData[Result].sDifficult:=tab.FieldAsString(8);
      MyOwnLife_aCacheData[Result].sTerrain:=tab.FieldAsString(9);
      //Log('sDifficult='+MyOwnLife_aCacheData[Result].sDifficult+', sTerrain='+MyOwnLife_aCacheData[Result].sTerrain);
      MyOwnLife_aCacheData[Result].sSize:=tab.FieldAsString(10);
      MyOwnLife_aCacheData[Result].sElevation:=tab.FieldAsString(11);
      //Log('sSize='+MyOwnLife_aCacheData[Result].sSize+', sElevation='+MyOwnLife_aCacheData[Result].sElevation);
      MyOwnLife_aCacheData[Result].dPublic:=IntToDate(tab.FieldAsString(12));//pokud nejsou logy v DB muze to byt chybne
      MyOwnLife_aCacheData[Result].nFoundIt:=tab.FieldAsInteger(13);
      //pocet nalezu jen pro neeventove kese, pro eventy se graf nekresli
      if(MyOwnLife_aCacheData[Result].nFoundIt>MyOwnLife_iMax) and (not MyOwnLife_aCacheData[Result].bEvent) then
        MyOwnLife_iMax:=MyOwnLife_aCacheData[Result].nFoundIt;
      MyOwnLife_aCacheData[Result].nDNF:=tab.FieldAsInteger(14);
      //MyOwnLife_aCacheData[Result].nFP:=StrToInt(tab.FieldAsString(15));   //takto to mel drak
      MyOwnLife_aCacheData[Result].nFP:=tab.FieldAsInteger(15);
      //Log('nFoundIt='+IntToStr(MyOwnLife_aCacheData[Result].nFoundIt)+', dPublic='+DateToStr(MyOwnLife_aCacheData[Result].dPublic)
      //  +', nDNF='+IntToStr(MyOwnLife_aCacheData[Result].nDNF)+', nFP='+IntToStr(MyOwnLife_aCacheData[Result].nFP));
      MyOwnLife_aCacheData[Result].sFTF:=UtfToAnsi(tab.FieldAsString(16));
      MyOwnLife_aCacheData[Result].sSTF:=UtfToAnsi(tab.FieldAsString(17));
      MyOwnLife_aCacheData[Result].sTTF:=UtfToAnsi(tab.FieldAsString(18));
      //Log('FTF='+MyOwnLife_aCacheData[Result].sFTF+', STF='+MyOwnLife_aCacheData[Result].sSTF
      //  +', TTF='+MyOwnLife_aCacheData[Result].sTTF);
      MyOwnLife_aCacheData[Result].nGcVoteCount:=tab.FieldAsInteger(19);
      MyOwnLife_aCacheData[Result].fGcVoteMedian:=StrToFl(tab.FieldAsString(20));
      //Log('nGcVoteCount='+IntToStr(MyOwnLife_aCacheData[Result].nGcVoteCount)+', fGcVoteMedian='+FloatToStr(MyOwnLife_aCacheData[Result].fGcVoteMedian));
      //dopocitame nektere hodnoty
      MyOwnLife_aCacheData[Result].nExistDays:=
        //1+Trunc(MyOwnLife_aCacheData[Result].dArchive-MyOwnLife_aCacheData[Result].dCreate);
        1+Trunc(MyOwnLife_aCacheData[Result].dArchive-MyOwnLife_aCacheData[Result].dPublic);
      //Log('nExistDays='+IntToStr(MyOwnLife_aCacheData[Result].nExistDays));
      //doplnime pocet logu
      MyOwnLife_aCacheData[Result].nLogs:=GEOGET_DB.GetTableValue(
        'SELECT count(gs_logid) FROM geolog WHERE id='''+MyOwnLife_aCacheData[Result].sId+'''');
      //doplnime datum posledniho nalezu
      MyOwnLife_aCacheData[Result].dLastFound:=IntToDate(IntToStr(
        GEOGET_DB.GetTableValue(
          'SELECT max(dt) FROM geolog WHERE id='''+MyOwnLife_aCacheData[Result].sId+''''
          +'  AND (type=''Found it'' OR type=''Attended'' OR type=''Webcam Photo Taken'')')
        ));
      MyOwnLife_aCacheData[Result].nWillAttend:=GEOGET_DB.GetTableValue(
        'SELECT count(gs_logid) FROM geolog WHERE id='''+MyOwnLife_aCacheData[Result].sId+''''
          +'  AND type=''Will Attend''');

      Inc(Result);
      tab.Next();
    end;
  finally
    tab.Free();
  end;
  Log('Pocet pripravenych kesi: '+IntToStr(Result));
end;

procedure MyOwnLife_PrepareLogs;    //pripravi pocty slov z logu pro vsechny kese
var i,n:integer;
    sSql,s:string;
    tab:TSQLiteTable;
begin
  StatorBusy(_('Counting words in logs ...'));
  for i:=Low(MyOwnLife_aCacheData) to High(MyOwnLife_aCacheData) do begin
    //precteme vsechny nalezove logy pro zadanou kes
    //asi by bylo spravne pocitat vsechny logy, ne jen nalezove!!!!
    sSql:='SELECT dt, logtext, finder, gs_logid '
      +'FROM geolog WHERE id='''+MyOwnLife_aCacheData[i].sId+''''
      +'  AND (type=''Found it'' OR type=''Attended'' OR type=''Webcam Photo Taken'') ORDER BY dt ASC';
    tab:=GEOGET_DB.GetTable(sSql, false);
    try
      while (not tab.Eof) do begin
        s:=UtfToAnsi(zDecompress(tab.FieldAsBlobText(1)));
        n:=MyOwnLife_CountWords(s);
        if(n>MyOwnLife_aCacheData[i].nWordsMax) then begin
          MyOwnLife_aCacheData[i].nWordsMax:=n;
          MyOwnLife_aCacheData[i].sWordsMaxFinder:=UtfToAnsi(tab.FieldAsBlobText(2));
          MyOwnLife_aCacheData[i].sWordsMaxLID:=tab.FieldAsBlobText(3);
        end;
        if(n<MyOwnLife_aCacheData[i].nWordsMin) then begin
          MyOwnLife_aCacheData[i].nWordsMin:=n;
          MyOwnLife_aCacheData[i].sWordsMinFinder:=UtfToAnsi(tab.FieldAsBlobText(2));
          MyOwnLife_aCacheData[i].sWordsMinLID:=tab.FieldAsBlobText(3);
        end;
        MyOwnLife_aCacheData[i].nWordsTotal:=MyOwnLife_aCacheData[i].nWordsTotal+n;
        tab.Next();
      end;
    finally
      tab.Free();
    end;
    if(MyOwnLife_aCacheData[i].nWordsMax=0) then begin
      Log(_('WARN: number of LogWords=0 for')+' '+MyOwnLife_aCacheData[i].sId
        +', '+_('start Logator plugin!'));
    end;
    //Log('IIIIII Name='+MyOwnLife_aCacheData[i].sName
    //  +': suma='+IntToStr(+MyOwnLife_aCacheData[i].nWordsTotal)
    //  +': min='+IntToStr(+MyOwnLife_aCacheData[i].nWordsMin)
    //  +': max='+IntToStr(+MyOwnLife_aCacheData[i].nWordsMax)
    //  );
  end;
end;

function MyOwnLife_Buttons(sMainItem:string):string;
var i:integer;
    s:string;
begin
  Result:='<div style="text-align:center; margin:15px;">'+CRLF
    +'<button id="'+sMainItem+'Show" style="cursor:pointer; margin:2px 2px 5px 2px; " onclick="';
  //zobrazeni vsech kesi
  for i:=Low(MyOwnLife_aCacheData) to High(MyOwnLife_aCacheData) do begin
    s:=MyOwnLife_GetIdElement(sMainItem,i);
    Result:=Result+'document.getElementById('''+s+''').style.display=''''; ';
  end;
  //doplnime zakazani tohoto tlacitka a povoleni druheho
  Result:=Result
    +'document.getElementById('''+sMainItem+'Show'').style.color='''+COLOR_BUT_INACTIVE_TXT+'''; '
    +'document.getElementById('''+sMainItem+'Hide'').style.color=''black''; '
    +'return false;">'+_('Show all caches')+'</button>'+CRLF;

  //zobrazeni kesi, ktere nejsou archivovane
  Result:=Result+'<button id="'+sMainItem+'Hide" style="cursor: pointer; color:'+COLOR_BUT_INACTIVE_TXT+';" onclick="';
  //zobrazeni vsech kesi
  for i:=Low(MyOwnLife_aCacheData) to High(MyOwnLife_aCacheData) do begin
    s:=MyOwnLife_GetIdElement(sMainItem,i);
    Result:=Result+'document.getElementById('''+s+''').style.display=';
    if(MyOwnLife_aCacheData[i].bArchived) then Result:=Result+'''none'';'
    else Result:=Result+''''';'
  end;
  //doplnime zakazani tohoto tlacitka a povoleni druheho
  Result:=Result
    +'document.getElementById('''+sMainItem+'Show'').style.color=''black''; '
    +'document.getElementById('''+sMainItem+'Hide'').style.color='''+COLOR_BUT_INACTIVE_TXT+'''; '
    +'return false;">'+_('Hide archived caches')+'</button>'+CRLF;
  Result:=Result+'</div>'+ CRLF;
end;

procedure MyOwnLife_GraphCache(idx:integer; aPar:TStringList);
var 
  tab: TSqliteTable;
  iMax,iMaxY,iYearMin,iYearMax,iMonthMin,iMonthMax,iDayMin,iDayMax,iCh,iChx:integer;
  s,ss,sY,sYz,sT,sC,sSql,sGraph,sEY,sEValue,sEOption,sEId,sDisable,sArchive,sYearly,sXaxis,sEValueNP,sYearlyNP:string;
  sColorNormal,sColorDisable,sColorArchive,sColorYearly,sColorTotal,sDf,xMin,xMax,sYY:string;
  bPub:boolean;
begin
	// Yearly stat show Yes/No
	sYY:='false';
	if UpperCase(aPar.Values['Yearly'])='ANO' then sYY:='true';
  // rozsahy vodorovne osy grafu
  s:=aPar.Values['Xscale'];
  sXaxis:=Fetch(s,'-');
  case sXaxis of
  'All':  begin
            iYearMin:=StrToInt(FormatDateTime('yyyy',MyOwnLife_dGraphFrom));
            iMonthMin:=StrToInt(FormatDateTime('m',MyOwnLife_dGraphFrom));
						iDayMin:=StrToInt(FormatDateTime('d',MyOwnLife_dGraphFrom));
          end;
  else {'One'}  begin
            iYearMin:=StrToInt(FormatDateTime('yyyy',MyOwnLife_aCacheData[idx].dCreate));
            iMonthMin:=StrToInt(FormatDateTime('m',MyOwnLife_aCacheData[idx].dCreate));
						iDayMin:=StrToInt(FormatDateTime('d',MyOwnLife_aCacheData[idx].dCreate));
          end;
  end;
  // konec vodorovn osy grafu
  s:=aPar.Values['EndOfGraph'];
  ss:=Fetch(s,'-');
  case ss of
  'EndOfLife':  begin
            iYearMax:=StrToInt(FormatDateTime('yyyy',MyOwnLife_aCacheData[idx].dLastFound));
            iMonthMax:=StrToInt(FormatDateTime('m',MyOwnLife_aCacheData[idx].dLastFound));
						iDayMax:=StrToInt(FormatDateTime('d',MyOwnLife_aCacheData[idx].dLastFound));
            if MyOwnLife_aCacheData[idx].bArchived and (MyOwnLife_aCacheData[idx].dArchive>MyOwnLife_aCacheData[idx].dLastFound) then begin
              iYearMax:=StrToInt(FormatDateTime('yyyy',MyOwnLife_aCacheData[idx].dArchive));
              iMonthMax:=StrToInt(FormatDateTime('m',MyOwnLife_aCacheData[idx].dArchive));
							iDayMax:=StrToInt(FormatDateTime('d',MyOwnLife_aCacheData[idx].dArchive));
            end;
          end;
  else {'Today'}  begin
            iYearMax:=StrToInt(FormatDateTime('yyyy',Now()));
            iMonthMax:=StrToInt(FormatDateTime('m',Now()));
						iDayMax:=StrToInt(FormatDateTime('d',Now()));
          end;
  end;
	xMin := IntToStr(iYearMin)+'-'+IntToStr(iMonthMin)+'-'+IntToStr(iDayMin);
	xMax := IntToStr(iYearMax)+'-'+IntToStr(iMonthMax)+'-'+IntToStr(iDayMax);
	
  // vytvooen zkladnch stringlistu pro data grafu
  iMax:=0;iMaxY:=0;iCh:=0;iChx:=0;sEValue:='';sDisable:='';sArchive:='';sYearly:='';sDf:='E';sYz:='';bPub:=false;
	sEValueNP:='';sYearlyNP:='';
	//Barvy
  //sColorNormal:=ColorToRGBA(GlobalVarGet('ColorGraphCacheTraditional',false),'0.4');
  sColorNormal:=ColorToRGBA('ababab','0.8');
  sColorDisable:=ColorToRGBA('ab0000','0.8');
  sColorArchive:=ColorToRGBA('000000','0.5');
	sColorYearly:=ColorToRGBA(GlobalVarGet('ColorGraphCacheUnknown',false),'1');
	sColorTotal:=ColorToRGBA(GlobalVarGet('ColorTableScaleAvgBkg',false),'1');
  // hodnoty z logu vybrane vlastni kese naplnime do poli
  sSql:='SELECT dt lDate, type LogType, count(*) Count '
       +'FROM geolog WHERE id='''+MyOwnLife_aCacheData[idx].sId+''' '
       +'GROUP BY lDate, LogType '
       +'ORDER BY dt,gs_logid ASC';
  tab:=Geoget_DB.GetTable(sSql,false);
  try
    while not tab.eof do begin
			//
      sT:=tab.FieldByName['LogType'];
      sC:=tab.FieldByName['Count'];
			if not ((sT = 'Write note') or (sT = 'Didn''t find it') or (sT = 'Update Coordinates') 
					or (sT = 'Needs Maintenance') or (sT = 'Owner Maintenance') or (sT='Needs Archived')) then begin
				//
				//{$ifdef DEBUG_HELPER}LDH('Message:'+sT);{$endif}
				//
				s:=tab.FieldByName['lDate'];
				ss:=Copy(s,1,4)+'-'+Copy(s,5,2)+'-'+Copy(s,7,2)	//Datum
				sY:=Copy(s,1,4)																	//jen rok kvli ronmu
				//
				if sDisable = '' then sDisable:=sDisable+'['''+ss+''',0],';	//prvn bod Disable
				if sArchive = '' then sArchive:=sArchive+'['''+ss+''',0],';	//prvn bod Archive
				if bPub and (sYearly = '') then sYz:=sY;																//Prvn rok
				if not bPub and (sYearlyNP = '') then sYz:=sY;															//Prvn rok
				//sEValue:=sEValue+'['''+ss+''',';
				//sYearly:=sYearly+'['''+ss+''',';
				//Vbr co je co
				case sT of
				'Publish Listing': begin
											//{$ifdef DEBUG_HELPER}LDH('PUBLISHED:'+ss+' - '+sYz+'/'+sY);{$endif}				
											bPub:=true;		//Publikovno
											if sXaxis <> 'All' then xMin := ss;		//Uprav kraj statistiky
											//sEValue:=sEValue+'['''+ss+''',0],';
											//sYearly:=sYearly+'['''+ss+''',0],';
										end;
				'Found it',
				'Webcam Photo Taken':	begin
											//{$ifdef DEBUG_HELPER}LDH('FOUND:'+ss+' - '+sYz+'/'+sY);{$endif}				
											iMax:=iMax+StrToInt(sC);
											if sYz <> sY then begin	//Mn se rok			
												//{$ifdef DEBUG_HELPER}LDH('ROK:'+sYz+'/'+sY);{$endif}
												sYz := sY;
												//
												if bPub then begin	//Existuje publikace
													sYearly:=sYearly+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearly:=sYearly+'['''+sY+'-01-01'',0],';
												end
												else begin					//Neexistuje publikace
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'',0],';
												end;
												if Copy(s,5,4) <> '0101' then begin //Nen 01.01
													//Vyrob jednoho 01.01 navc
													sEValue:=sEvalue+'['''+sY+'-01-01'','+IntToStr(iMax)+'],';
													if sDf = 'D' then sDisable:=sDisable+'['''+sY+'-01-01'','+IntToStr(iMax)+'],'; //nlez pi disable
													if sDf = 'A' then sArchive:=sArchive+'['''+sY+'-01-01'','+IntToStr(iMax)+'],'; //nlez pi archive
												end;
												//
												iMaxY := StrToInt(sC);
											end
											else iMaxY := iMaxY + StrToInt(sC);
											if sDf = 'D' then sDisable:=sDisable+'['''+ss+''','+IntToStr(iMax)+'],'; //nlez pi disable
											if sDf = 'A' then sArchive:=sArchive+'['''+ss+''','+IntToStr(iMax)+'],'; //nlez pi archive
											end;
				'Temporarily Disable Listing':  begin
											//{$ifdef DEBUG_HELPER}LDH('DISABLE:'+ss+' - '+sYz+'/'+sY);{$endif}
											if sDf <> 'D' then sDisable:=sDisable+'['''+ss+''',0],';	//prvn bod Disable
											sDisable:=sDisable+'['''+ss+''','+IntToStr(iMax)+'],';	//prvn bod Disable
											if sYz <> sY then begin
												sYz := sY;
												if bPub then begin
													sYearly:=sYearly+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearly:=sYearly+'['''+sY+'-01-01'',0],';
												end
												else begin
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'',0],';
												end;
												iMaxY := 0;
											end;
											iChx:=iCh;	//pechodov sekvence
											sDf:='D';	//koncov sek
											end;
				'Enable Listing':	begin
											//{$ifdef DEBUG_HELPER}LDH('ENABLE:'+ss+' - '+sYz+'/'+sY);{$endif}
											if sDf = 'D' then begin
												sDisable:=sDisable+'['''+ss+''','+IntToStr(iMax)+'],';	//posledn bod Disable
												sDisable:=sDisable+'['''+ss+''',0],';									//prvn bod Enable
											end;
											if sDf = 'A' then begin
												sArchive:=sArchive+'['''+ss+''','+IntToStr(iMax)+'],';	//posledn bod Disable
												sArchive:=sArchive+'['''+ss+''',0],';									//prvn bod Enable
											end;
											if sYz <> sY then begin
												sYz := sY;
												if bPub then begin
													sYearly:=sYearly+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearly:=sYearly+'['''+sY+'-01-01'',0],';
												end
												else begin
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'',0],';
												end;
												iMaxY := 0;
											end;
											iChx:=iCh;	//pechodov sekvence
											sDf:='E'; 	//koncov sek
											end;
				'Archive':		begin
											//{$ifdef DEBUG_HELPER}LDH('ARCHIVE:'+ss+' - '+sYz+'/'+sY);{$endif}
											if sDf = 'D' then begin
												sDisable:=sDisable+'['''+ss+''','+IntToStr(iMax)+'],';	//posledn bod Disable
												sDisable:=sDisable+'['''+ss+''',0],';									//prvn bod Enable
											end;
											if sDf <> 'A' then sArchive:=sArchive+'['''+ss+''',0],';		//prvn bod Enable
											sArchive:=sArchive+'['''+ss+''','+IntToStr(iMax)+'],';			//posledn bod Disable
											if sYz <> sY then begin
												sYz := sY;
												if bPub then begin
													sYearly:=sYearly+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearly:=sYearly+'['''+sY+'-01-01'',0],';
												end
												else begin
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'','+IntToStr(iMaxY)+'],';
													sYearlyNP:=sYearlyNP+'['''+sY+'-01-01'',0],';
												end;
												iMaxY := 0;
											end;
											iChx:=iCh;	//pechodov sekvence
											sDf:='A'; 	//koncov sek
											end;
				'Unarchive':	begin
											//{$ifdef DEBUG_HELPER}LDH('UNARCHIVE:'+ss+' - '+sYz+'/'+sY);{$endif}
											if sDf = 'A' then begin
												sArchive:=sArchive+'['''+ss+''','+IntToStr(iMax)+'],';	//posledn bod Archive
												sArchive:=sArchive+'['''+ss+''',0],';									//prvn bod Enable
											end;
											iChx:=iCh;	//pechodov sekvence
											sDf:='E'; 	//koncov sek
											end;
				end;
				if bPub then begin
					sEValue:=sEValue+'['''+ss+''','+IntToStr(iMax)+'],';
					sYearly:=sYearly+'['''+ss+''','+IntToStr(iMaxY)+'],';
				end
				else begin
					sEValueNP:=sEValueNP+'['''+ss+''','+IntToStr(iMax)+'],';
					sYearlyNP:=sYearlyNP+'['''+ss+''','+IntToStr(iMaxY)+'],';
				end;
				Inc(iCh);
			end;
			tab.Next();
		end;
		if not bPub then begin
			sEValue := sEvalueNP;
			sYearly := sYearlyNP;
		end;
		//Koncov datum
		s:=DateToStr(Now);
		ss:=Copy(s,7,4)+'-'+Copy(s,4,2)+'-'+Copy(s,1,2)
		if Pos(ss,sEValue)=0 then begin
			sEValue:=sEValue+'['''+ss+''','+IntToStr(iMax)+',],';
			sYearly:=sYearly+'['''+ss+''','+IntToStr(iMaxY)+',],';
			if sDf = 'D' then sDisable:=sDisable+'['''+ss+''','+IntToStr(iMax)+']'
			else sDisable:=sDisable+'['''+ss+''',0]';
			if sDf = 'A' then sArchive:=sArchive+'['''+ss+''','+IntToStr(iMax)+']'
			else sArchive:=sArchive+'['''+ss+''',0]';
		end;
	//-------------
  finally
    tab.Free();
  end;
	// rozsahy svisl osy grafu
  s:=aPar.Values['Yscale'];
  ss:=Fetch(s,'-');
  case ss of
  'All':  iMax:=MyOwnLife_iMax;
  end;
  iMax:=Round(1.05*iMax);         // prava celkovho rozsahu, aby byly videt praporky
  if(iMax=0) then begin          // pokud nejsou dn FI logy
    AddError(_('WARN: Missing Found It logs for')+' '+MyOwnLife_aCacheData[idx].sId+'. '+_('You must get logs from geocaching.com!'));
    iMax:=1;
  end;
	//
  sEValue:=Copy(sEValue,1,Length(sEValue)-1);
	sYearly:=Copy(sYearly,1,Length(sYearly)-1);
  //yAxes
  sEY:='yAxis:['+CRLF
	+'  {type: ''value'','+CRLF
	+'   name:'''+_('Findings')+''','+CRLF
	+'   position:''left'','+CRLF
	+'   min:0, max:'+IntToStr(iMax)+','+CRLF	
	+'   nameGap:8,'+CRLF
	+'   nameTextStyle:{fontSize:10},'+CRLF
	+'   axisLabel:{fontSize:10,color:'+sColorTotal+'},'+CRLF
	+'   splitLine:{lineStyle:{color:'+sColorTotal+'}},'+CRLF
	+'   axisLine:{lineStyle:{color:'+sColorTotal+'}}'+CRLF
	+'  },'+CRLF
	+'  {type:''value'','+CRLF
	+'   position:''right'','+CRLF
	+'   nameGap:8,'+CRLF
	+'   name:'''+_('Yearly')+''','+CRLF
	+'   splitArea:{show: true},'+CRLF
	+'   nameTextStyle:{fontSize:10, color:'+sColorYearly+'},'+CRLF
	+'   axisLabel:{fontSize:10, color:'+sColorYearly+'}'+CRLF
	+'  }],';
	//option
  sEOption:='option={'+CRLF
  +'  dataZoom: [{type: ''inside''}, {type: ''slider''}],'+CRLF
  +'  tooltip:{show:true, trigger:''axis'', formatter: function(par){'+CRLF
	+'  p0 = par[0];'+CRLF
  +'  idxN=-1;'+CRLF
  +'  idxR=-1;'+CRLF
  +'  for(i=0; i<3; i++){'+CRLF
  +'    if(par[i]){'+CRLF
  +'      if(par[i].seriesIndex==0) idxN=i;'+CRLF
  +'      if(par[i].seriesIndex==3) idxR=i;'+CRLF
  +'    }'+CRLF
  +'  }'+CRLF
  +'  sN=''''; sR='''';'+CRLF
  +'  if(idxN>=0) sN=par[idxN].marker+'' ''+par[idxN].seriesName+'': ''+par[idxN].value[1]+''<br/>'';'+CRLF
  +'  if(idxR>=0) sR=par[idxR].marker+'' ''+par[idxR].seriesName+'': ''+par[idxR].value[1]+''<br/>'';'+CRLF
	+'  a=par[0].value[0].split(''-'');'+CRLF
  +'  d=a[2]+''.''+a[1]+''.''+a[0];'+CRLF
  +'  s=''<p style="text-align:center; margin:0px;">''+d+''</p>'''+CRLF
  +'    +''<p style="text-align:left; margin:0px;">'''+CRLF
  +'    +sN+sR'+CRLF
  +'    +''</p>'';'+CRLF
  +'  return s;}'+CRLF
  +'  },'+CRLF
  +'  grid:{ left:5, right:5, bottom:40, top:40, containLabel:true },'+CRLF
  +'  xAxis:[{type: ''time'','+CRLF
  +'    minInterval: 3600 * 1000 * 24,'+CRLF
	+'		min:'''+xMin+''','+CRLF
	+'		max:'''+xMax+''','+CRLF
	+'    axisLabel:{fontSize:10,'+CRLF
	+'		  color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+','+CRLF
	+'		  formatter: function(val,idx){'+CRLF
	+'      s=echarts.format.formatTime("dd.MM.\r\nyyyy", val);'+CRLF
  +'      return(s);'+CRLF
  +'    }},'+CRLF
  +'    splitLine: {'+CRLF
  +'      lineStyle: {color:'+ColorToRGBA(GlobalVarGet('ColorTableScaleAvgBkg',false),'0.6')+'}'+CRLF
  +'    }'+CRLF
  +'  }],'+CRLF
  +'  '+sEY+CRLF
	+'  legend:{data:['''+_('Found')+''','''+_('Disabled')+''','''+_('Archived')+''','''+_('Yearly')+'''],'+CRLF
	+'      selected:{'''+_('Found')+''': true,'''+_('Disabled')+''': true,'''+_('Archived')+''': true,'''+_('Yearly')+''':'+sYY+'},'+CRLF
	+'     textStyle:{fontSize:8, color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+'}, itemGap:7},'+CRLF
  +'  series:[{name:'''+_('Found')+''', color:'+sColorNormal+', symbolSize:1, symbol:"circle", yAxisIndex:0, data:['+sEValue+'],'+CRLF
  +'  type: ''line'','+CRLF
  +'  areaStyle: {}'+CRLF
  +'  },'+CRLF
	+'  {name:'''+_('Disabled')+''',color:'+sColorDisable+', symbolSize:1, symbol:"circle", yAxisIndex:0, data:['+sDisable+'],'+CRLF
  +'  type: ''line'','+CRLF
  +'  areaStyle: {}'+CRLF
  +'  },'+CRLF
	+'  {name:'''+_('Archived')+''',color:'+sColorArchive+', symbolSize:1, symbol:"circle", yAxisIndex:0, data:['+sArchive+'],'+CRLF
  +'  type: ''line'','+CRLF
  +'  areaStyle: {}'+CRLF
  +'  },'+CRLF
	+'  {name:'''+_('Yearly')+''',color:'+sColorYearly+', symbolSize:1, symbol:"circle", yAxisIndex:1, data:['+sYearly+'],'+CRLF
  +'  type: ''line'','+CRLF
  +'  lineStyle:{type:''dotted''}'+CRLF
  +'  }'+CRLF
	+'  ],};';
  //-----------------------------
  // finln sestaven URL grafu
	//ECharts --------------------
	sEId:='ID'+IntToStr(iItem)+'mol'+IntToStr(idx);
	sGraph:='<div id="'+sEId+'" style="width:'+IntToStr(MyOwnLife_GraphWidth)+'px; height:'+IntToStr(MyOwnLife_GraphHeight)+'px;"></div>'+CRLF
	//+'%StyleTableBottomTextStartHtml%'+
	//+'<i>'+_('You can zoom by scrolling or using sliders under the chart.')+'</i>'     
	//+'%StyleTableBottomTextEndHtml%'+CRLF 
	+'<script type="text/javascript">'+CRLF
	+'  var chart_'+sEId+'=echarts.init(document.getElementById('''+sEId+'''));'+CRLF
	+sEOption
	//vytvoreni grafu s potrebnymi parametry a definovanym tematem
	+'  chart_'+sEId+'.setOption(option);'+CRLF
	+'</script>'+CRLF+CRLF;		 
	//vse uz je prevedeno do UTF a entit, neprevadet znovu
	//sGraph:=AnsiToUtf(sGraph);  // prevedeme na utf
{$ifdef DEBUG_HELPER}
	LDH('GraphData:'+CRLF+sGraph);
{$endif}
	Output(sGraph);
	// klid
end;

procedure MyOwnLife_GraphEvent(idx:integer; aPar:TStringList);
var 
  s,sSql,sGraph,sEOption,sEId:string;
  iWill,iAtt,iBoth,iMax:integer;
begin
	iMax := 0;
  // zkladn hodnoty pro Bar graf
  iWill:=MyOwnLife_aCacheData[idx].nWillAttend;
	if iWill > iMax then iMax := iWill;
  iAtt:=MyOwnLife_aCacheData[idx].nFoundIt;
	if iAtt > iMax then iMax := iAtt;
  // poet nicku, kte logovali Will Attend i Attended
  s:=MyOwnLife_aCacheData[idx].sID;
  sSql:='SELECT COUNT(finder) FROM geolog WHERE finder IN (SELECT finder FROM geolog '
       +'WHERE type=''Will Attend'' AND id='''+s+''') AND type=''Attended'' AND id='''+s+'''';
  iBoth:=Geoget_DB.GetTableValue(sSql);
	if iBoth > iMax then iMax := iBoth;
	iMax := Trunc(Round(1.1*iMax)/10.0+1)*10;
	// finln sestaven skriptu grafu ECHART
	sEId:='ID'+IntToStr(iItem)+'mol'+IntToStr(idx);
	sEOption:='option = {'+CRLF
	+'  xAxis: {'+CRLF
	+'	 axisLabel:{color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+'},'+CRLF
	+'   type: ''category'','+CRLF
	+'   nameTextStyle:{fontSize:10,color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+'},'+CRLF
	+'   data: ['''+_('Will Attend')+''', '''+_('Intersection')+''', '''+_('Attended')+''']'+CRLF
	+'  },'+CRLF
	+'  yAxis: {'+CRLF
	+'	 axisLabel:{color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+'},'+CRLF
	+'   type: ''value'','+CRLF
	+'   name:'''+_('Cachers')+''','+CRLF
	+'   nameGap:8,'+CRLF	
	+'   min:0, max:'+IntToStr(iMax)+','+CRLF
	+'   nameTextStyle:{fontSize:10,color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+'}'+CRLF
	+'  },'+CRLF
	+' tooltip : {trigger: ''item'', axisPointer : {type : ''shadow''},'+CRLF
  +' formatter: function(par){'+CRLF
  +'   s='''';'+CRLF
  +'   if (par.name == '''+_('Will Attend')+''') {s = s+'''+_('WA logs')+'''}'+CRLF
  +'   if (par.name == '''+_('Intersection')+''') {s = s+'''+_('logged Will attend <br>and Attended')+'''}'+CRLF
  +'   if (par.name == '''+_('Attended')+''') {s = s+'''+_('came')+'''}'+CRLF
  +'   s = par.marker + '' '' + par.value + '' '' + s;'+CRLF
  +'   return s;'+CRLF
 +'}},'+CRLF
	+' legend: {data: ['''+_('Will Attend')+''', '''+_('Intersection')+''', '''+_('Attended')+'''], textStyle:{fontSize:10, color:'+ColorToRGBA(GlobalVarGet('ColorMainText',false),'1')+'}},'+CRLF
	+' series: [{name:'''+_('Will Attend')+''',type:''bar'',stack: ''stack'', label: {normal: {show: true, position: ''inside''}}, data:['+IntToStr(iWill)+',null,null]},'+CRLF
	+'					{name:'''+_('Intersection')+''',type:''bar'',stack: ''stack'', label: {normal: {show: true, position: ''inside''}}, data:[null,'+IntToStr(iBoth)+',null]},'+CRLF
	+'					{name:'''+_('Attended')+''',type:''bar'',stack: ''stack'', label: {normal: {show: true, position: ''inside''}}, data:[null,null,'+IntToStr(iAtt)+']}'+CRLF
	+' ]'+CRLF
	+'};'+CRLF;
	//
	sGraph:='<div id="'+sEId+'" style="width:'+IntToStr(MyOwnLife_GraphWidth)+'px; height:'+IntToStr(MyOwnLife_GraphHeight)+'px;"></div>'+CRLF
	+'<script type="text/javascript">'+CRLF
	+'  var chart_'+sEId+'=echarts.init(document.getElementById('''+sEId+'''));'+CRLF
	+sEOption
	//vytvoreni grafu s potrebnymi parametry a definovanym tematem
	+'  chart_'+sEId+'.setOption(option);'+CRLF
	+'</script>'+CRLF+CRLF;	
  //vse uz je prevedeno do UTF a entit, neprevadet znovu
  //sGraph:=AnsiToUtf(sGraph);  // prevedeme na UTF
  Output(sGraph);
end;

function MyOwnLife_MakeLinkToProfile(sNicks:string):string;
var s:string;
begin
  Result:='';
  while(sNicks<>'') do begin
    s:=Fetch(sNicks,'|');
    if(Result<>'') then Result:=Result+', ';
    Result:=Result+'<a href="https://www.geocaching.com/profile/default.aspx?u='
      +EncodeUrlElement(AnsiToUtf(s))+'" target="_blank">'+s+'</a>';
  end;
end;

procedure MyOwnLife_TableCache(idx:integer; aPar:TStringList);
var s:string;
    iWold:integer;
    bOdd:boolean;
begin
  iWold:=iWidth;
  iWidth:=iWidth-MyOwnLife_GraphWidth-10;
  GlobalVarSet('Width',IntToStr(iWidth));
  s:='<table style="margin:0px auto 0px auto; '+GlobalVarGet('StyleTableNum',false)+'">'+CRLF;
  
  bOdd:=false;
  //odlisime logy pro normalni a eventove kese
  if(MyOwnLife_aCacheData[idx].bEvent) then begin
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Will Attend')+'</td><td style="%StyleTableNumTd%"><b>'
      +IntToStr(MyOwnLife_aCacheData[idx].nWillAttend)+'</b></td></tr>';
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Attended')+'</td><td style="%StyleTableNumTd%"><b>'
      +IntToStr(MyOwnLife_aCacheData[idx].nFoundIt)+'</b></td></tr>';
  end
  else begin
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Found it')+'</td><td style="%StyleTableNumTd%"><b>'
      +IntToStr(MyOwnLife_aCacheData[idx].nFoundIt)+'</b></td></tr>';
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('DNF')+'</td><td style="%StyleTableNumTd%"><b>'
      +IntToStr(MyOwnLife_aCacheData[idx].nDNF)+'</b></td></tr>';
  end;
  if(not MyOwnLife_aCacheData[idx].bEvent) then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Cache success')+'</td><td style="%StyleTableNumTd%"><b>'
      +FormatFloatDot('# ##0.00',(0.0+MyOwnLife_aCacheData[idx].nFoundIt)/MyOwnLife_aCacheData[idx].nExistDays)
      +'</b> '+_('finds per day')+'</td></tr>';
  if(not MyOwnLife_aCacheData[idx].bEvent) then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Published')+'</td><td style="%StyleTableNumTd%"><b>'+DateToStr(MyOwnLife_aCacheData[idx].dPublic)
      +'</b> ('+_('Created:')+' <b>'+DateToStr(MyOwnLife_aCacheData[idx].dCreate)+'</b>)</td></tr>'
  else
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Event date')+'</td><td style="%StyleTableNumTd%"><b>'+DateToStr(MyOwnLife_aCacheData[idx].dCreate)
      +'</b></td></tr>';
  if(not MyOwnLife_aCacheData[idx].bEvent) then begin
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Last found')+'</td><td style="%StyleTableNumTd%"><b>'+DateToStr(MyOwnLife_aCacheData[idx].dLastFound)+'</b></td></tr>';
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Days hidden')+'</td><td style="%StyleTableNumTd%"><b>'+IntToStr(MyOwnLife_aCacheData[idx].nExistDays)
      +'</b> '+_('days')+'</td></tr>';
  end;
  if(MyOwnLife_aCacheData[idx].nLogs=0) then MyOwnLife_aCacheData[idx].nLogs:=1;
  s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
    +'<td style="%StyleTableNumLabelTd%">'+_('Logs')+'</td><td style="%StyleTableNumTd%"><b>'+IntToStr(MyOwnLife_aCacheData[idx].nLogs)+'</b>';
  //pokud neni zadny nalezovy log, nevypisujeme nic
  if(MyOwnLife_aCacheData[idx].nWordsTotal>0) then begin
    if(MyOwnLife_Param.bWordLogs) then
      s:=s+'; '+_('Average log length')+' '
        +'<b>'+FormatFloatDot('# ##0.00',(0.0+MyOwnLife_aCacheData[idx].nWordsTotal)/MyOwnLife_aCacheData[idx].nLogs)
        +'</b> '+_('words')
    else s:=s+'</td></tr>';
    if(MyOwnLife_Param.bWordLogs) then begin
      s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
        +'<td style="%StyleTableNumLabelTd%">'+_('Longest log')+'</td><td style="%StyleTableNumTd%">'
        +'<a href="https://www.geocaching.com/seek/log.aspx?LID='+MyOwnLife_aCacheData[idx].sWordsMaxLID+'" target="_blank">'
        +'<b>'+IntToStr(MyOwnLife_aCacheData[idx].nWordsMax)+'</b></a> '+_('words')
        +' ('+MyOwnLife_MakeLinkToProfile(MyOwnLife_aCacheData[idx].sWordsMaxFinder)+')'
        +'</td></tr>';
      s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
        +'<td style="%StyleTableNumLabelTd%">'+_('Shortest log')+'</td><td style="%StyleTableNumTd%">'
        +' <a href="https://www.geocaching.com/seek/log.aspx?LID='+MyOwnLife_aCacheData[idx].sWordsMinLID+'" target="_blank">'
        +'<b>'+IntToStr(MyOwnLife_aCacheData[idx].nWordsMin)+'</b></a> '+_('words')
        +' (<a href="https://www.geocaching.com/profile/default.aspx?u='+MyOwnLife_aCacheData[idx].sWordsMinFinder+'" target="_blank">'
        +MyOwnLife_aCacheData[idx].sWordsMinFinder+'</a>)'
        +'</td></tr>';
    end;
  end;
  if((MyOwnLife_Param.bFP) and (not MyOwnLife_aCacheData[idx].bEvent)) then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('Favorite points')+'</td><td style="%StyleTableNumTd%"><b>'
      //FP % trva dlouho a ma limit na pocet pristupu, nestoji to za tu "cenu"
      +IntToStr(MyOwnLife_aCacheData[idx].nFP)+'</b></td></tr>';
  if(aPar.Values['GCVote']='Yes') then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('GC Vote')+'</td><td style="%StyleTableNumTd%"><b>'
      +IntToStr(MyOwnLife_aCacheData[idx].nGCVoteCount)+'</b>'
      +' (median: <b>'+FormatFloatDot('0.0',MyOwnLife_aCacheData[idx].fGCVoteMedian)+'</b>)'
      +'</td></tr>';
{
    s:=s+'&nbsp;&nbsp;&nbsp;'+IntToStr(MyOwnLife_aCacheData[idx].nFP)+'  <img src="%IconLayoutFavorites%" />';
    +'&nbsp;&nbsp;&nbsp;'+IntToStr(MyOwnLife_aCacheData[idx].nGCVoteCount)+'  <img src="%IconLayoutGCVote%" />'
}  
  if(MyOwnLife_aCacheData[idx].sFTF<>'') then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('FTF')+'</td><td style="%StyleTableNumTd%"><b>'
      +MyOwnLife_MakeLinkToProfile(MyOwnLife_aCacheData[idx].sFTF)+'</b></td></tr>';
  if(MyOwnLife_aCacheData[idx].sSTF<>'') then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('STF')+'</td><td style="%StyleTableNumTd%"><b>'
      +MyOwnLife_MakeLinkToProfile(MyOwnLife_aCacheData[idx].sSTF)+'</b></td></tr>';
  if(MyOwnLife_aCacheData[idx].sTTF<>'') then
    s:=s+'  <tr style="%StyleTableNumTr%">'+CRLF
      +'<td style="%StyleTableNumLabelTd%">'+_('TTF')+'</td><td style="%StyleTableNumTd%"><b>'
      +MyOwnLife_MakeLinkToProfile(MyOwnLife_aCacheData[idx].sTTF)+'</b></td></tr>';
  s:=s+'</table>'+CRLF;
  Output(s);
  iWidth:=iWold;
  GlobalVarSet('Width',IntToStr(iWidth));
end;

function MyOwnLife_TableLogs(idx:integer):string;
var sSql:string;
    tab:TSQLiteTable;
begin
    Result:='<p style="text-align:center;">';
    GEOGET_DB.AddParamText(':id',MyOwnLife_aCacheData[idx].sId);
    sSql:='SELECT count(type), type FROM geolog where id=:id group by type order by count(type) desc '
    tab:=GEOGET_DB.GetTable(sSql, false);
    try
      while (not tab.Eof) do begin
        Result:=Result//+'<td>'
          +'<img src="%IconCacheLog'+tab.FieldAsString(1)+'%" title="'+tab.FieldAsString(1)+'" />'+tab.FieldAsString(0)
          //+'</td>'+CRLF;
          +'&nbsp;&nbsp;&nbsp;';
        tab.Next();
      end;
    finally
      tab.Free();
    end;
    Result:=Result+'</p>'+CRLF;
end;

procedure MyOwnLife_MonthYear(idx:integer; aPar:TStringList);
var s,sTab,sSql,sStyle:string;
    tab:TSQLiteTable;
    iM,iR,iRmin,iRmax,nR,i,n,iMax,nMaxRok,nMaxMes:integer;
    aFinds: array of integer;
begin
  //vytvori tabulku mesic/rok s poctem nalezu v kazdem policku
  //zjistime nejnizsi, nejvyssi rok a pocet roku do tabulky
  GEOGET_DB.AddParamText(':id',MyOwnLife_aCacheData[idx].sId);
  sSql:='SELECT distinct substr(dt,1,4) FROM geolog where id=:id and type=''Found it'' group by substr(dt,1,4) order by substr(dt,1,4) asc';
  tab:=GEOGET_DB.GetTable(sSql, false);
  try
    iRmin:=tab.FieldAsInteger(0);
    while (not tab.Eof) do begin
      iRmax:=tab.FieldAsInteger(0);
      tab.Next();
    end;
  finally
    tab.Free();
  end;
  nR:=iRmax-iRmin+1;
  iMax:=0;

  //nastavime velikost pole pro data
  SetLength(aFinds,(nR+1)*13);
  //nacteme pocty nalezu v jednotlivych mesicich
  GEOGET_DB.AddParamText(':id',MyOwnLife_aCacheData[idx].sId);
  sSql:='SELECT count(id), substr(dt,1,4), substr(dt,5,2) FROM geolog where id=:id and type=''Found it'' group by substr(dt,1,6) order by substr(dt,1,6) asc';
  tab:=GEOGET_DB.GetTable(sSql, false);
  try
    while (not tab.Eof) do begin
      i:=tab.FieldAsInteger(0);
      if(iMax<i) then iMax:=i;       //maximalni pocet nalezu za mesic
      iR:=tab.FieldAsInteger(1);
      iM:=tab.FieldAsInteger(2);
      aFinds[(iR-iRmin)*13+iM-1]:=i; //ulozime pocet nalezu v mesici
      aFinds[(iR-iRmin)*13+12]:=aFinds[(iR-iRmin)*13+12]+i; //pocitame nalezy v roce
      if(nMaxRok<aFinds[(iR-iRmin)*13+12]) then nMaxRok:=aFinds[(iR-iRmin)*13+12];
      aFinds[nR*13+iM-1]:=aFinds[nR*13+iM-1]+i; //pocitame nalezy v mesici
      if(nMaxMes<aFinds[nR*13+iM-1]) then nMaxMes:=aFinds[nR*13+iM-1];
      tab.Next();
    end;
  finally
    tab.Free();
  end;
  //ShowMessage('nMaxRok='+IntToStr(nMaxRok)+', nMaxMes='+IntToStr(nMaxMes)+', iMax='+IntToStr(iMax));
  //hlavicka tabulky
  sSql:=GlobalVarGet('ColumnHeaderYear',false); //udelame svisly text
  //if bCond then begin //udelame svisly text
  s:='';
  for i:=1 to Length(sSql) do begin
    s:=s+'&nbsp;'+sSql[i]+'&nbsp;<br />';
  end;
  //end;
  sTab:=sTab+'<br /><p style="text-align:center; font-weight:bold;">'+_('Finds overview')+'</p>'
    +'<table style="'+GlobalVarGet('StyleTableMatrix',false)+'">'+CRLF;
  sTab:=ReplaceString(sTab,'%Width%',IntToStr(iWidth)); //maximalni sirka tabulky
  //sTab:=ReplaceString(sTab,'%Width%','auto'); //minimalni rirka tabulky
  sTab:=sTab
    +'  <tr style="%StyleTableMatrixTr%"><th style="%StyleTableMatrixTh1%">&nbsp;</th><th colspan=14 style="%StyleTableMatrixTh1%">%MonthMonth%</th></tr>'+CRLF
    +'  <tr style="%StyleTableMatrixTr%">'
    +'    <th rowspan='+IntToStr(nR+2)+' style="%StyleTableMatrixTh1Down%">'+s+'</th>'
    +'    <th style="%StyleTableMatrixTh2%">&nbsp;</th>'
    +'    <th style="%StyleTableMatrixTh2%">%MonthJan%</th><th style="%StyleTableMatrixTh2%">%MonthFeb%</th>'
    +'    <th style="%StyleTableMatrixTh2%">%MonthMar%</th><th style="%StyleTableMatrixTh2%">%MonthApr%</th>'
    +'    <th style="%StyleTableMatrixTh2%">%MonthMa%</th><th style="%StyleTableMatrixTh2%">%MonthJun%</th>'
    +'    <th style="%StyleTableMatrixTh2%">%MonthJul%</th><th style="%StyleTableMatrixTh2%">%MonthAug%</th>'
    +'    <th style="%StyleTableMatrixTh2%">%MonthSep%</th><th style="%StyleTableMatrixTh2%">%MonthOct%</th>'
    +'    <th style="%StyleTableMatrixTh2%">%MonthNov%</th><th style="%StyleTableMatrixTh2%">%MonthDec%</th>'
    +'    <th style="%StyleTableMatrixTh2%">&sum;</th>'
    +'  </tr>'+CRLF;
  //vytvorime datove radky tabulky
  for iR:=0 to nR-1 do begin
    sTab:=sTab+'  <tr style="%StyleTableMatrixTr%"><th style="%StyleTableMatrixTh2%">'+IntToStr(iRmin+iR)+'</th>'+CRLF;
    for iM:=0 to 11 do begin
      i:=iR*13+iM;
      if(aFinds[i]=0) then sTab:=sTab+'<td style="%StyleTableMatrixTdEmpty%">&nbsp;</td>'+CRLF
      else begin
        if(aFinds[i]=iMax) and (GlobalVarGet('StyleTableMatrixTdMaxValueText',true)<>'') then begin
          sTab:=sTab+'    <td style="%StyleTableMatrixTdMaxValue% background:#'
            +ScaleColor(aFinds[i],0,iMax,GlobalVarGet('ColorTableScaleMinBkg',false),GlobalVarGet('ColorTableScaleAvgBkg',false),GlobalVarGet('ColorTableScaleMaxBkg',false))
            +'">'+IntToStr(aFinds[i])+'</td>'+CRLF;
        end
        else begin
          sTab:=sTab+'    <td style="%StyleTableMatrixTd% background:#'
            +ScaleColor(aFinds[i],0,iMax,GlobalVarGet('ColorTableScaleMinBkg',false),GlobalVarGet('ColorTableScaleAvgBkg',false),GlobalVarGet('ColorTableScaleMaxBkg',false))
            +'">'+IntToStr(aFinds[i])+'</td>'+CRLF;
        end;
      end;
    end;
    sStyle:=GlobalVarGet('StyleTableMatrixTh2',false);
    if(aFinds[iR*13+12]=nMaxRok) and (GlobalVarGet('StyleTableMatrixTh2Max',true)<>'')
      then sStyle:=GlobalVarGet('StyleTableMatrixTh2Max',false);
    sTab:=sTab+'  <th style="'+sStyle+'">'+IntToStr(aFinds[iR*13+12])+'</th>'+CRLF+'  </tr>'+CRLF;
    //sTab:=sTab+'  <th style="%StyleTableMatrixTh2%">'+IntToStr(aFinds[iR*13+12])+'</th></tr>'+CRLF;
  end;
  //doplnime soucty za jednotlive mesice
  sTab:=sTab+'  <tr style="%StyleTableMatrixTr%"><th style="%StyleTableMatrixTh2%">&sum;</th>'+CRLF;
  i:=0;
  for iM:=0 to 11 do begin
    i:=i+aFinds[nR*13+iM];
    sStyle:=GlobalVarGet('StyleTableMatrixTh2',false);
    if(aFinds[nR*13+iM]=nMaxMes) and (GlobalVarGet('StyleTableMatrixTh2Max',false)<>'StyleTableMatrixTh2Max')
      then sStyle:=GlobalVarGet('StyleTableMatrixTh2Max',false);
    sTab:=sTab+'<th style="'+sStyle+'">'+IntToStr(aFinds[nR*13+iM])+'</th>'+CRLF;
    //sTab:=sTab+'<th style="%StyleTableMatrixTh2%">'+IntToStr(aFinds[nR*13+iM])+'</th>'+CRLF
  end;
  sTab:=sTab+'<th style="%StyleTableMatrixTh2%">'+IntToStr(i)+'</th>'+CRLF;
  sTab:=sTab+'  </tr>'+CRLF;
  sTab:=sTab+'</table>'+CRLF;
  s:=aPar.Values['TextBottom'];
  if(s<>'') then begin
    //zjistime pocet kombinaci
    n:=0;
    for iR:=0 to nR-1 do begin
      for iM:=0 to 11 do begin
        if(aFinds[iR*13+iM]>0) then Inc(n);
      end;
    end;
    s:=ReplaceString(s,'%FoundCount%',IntToStr(n));
    s:=ReplaceString(s,'%AllCount%',IntToStr(12*(nR-1)));
    sTab:=sTab+'%StyleTableBottomTextStartHtml%'+s+'%StyleTableBottomTextEndHtml%'+CRLF;
 end;
 sTab:=sTab+'<br /><br />'+CRLF;
 Output(sTab);
 SetLength(aFinds,0);
end;

procedure MyOwnLife_OutputCache(idx:integer; sMainItem:string; aPar:TStringList);
var s,sOut:string;
begin
  //protoze je to tu pomale, zobrazime informaci o tom, co delame
  StatorBusy(IntToStr(idx)+'/'+IntToStr(High(MyOwnLife_aCacheData))+'  '+
    MyOwnLife_aCacheData[idx].sId+' - '+MyOwnLife_aCacheData[idx].sName);
  //zkonstruujeme nadpis
  s:='<div style="%StyleMyOwnCacheTitle1%">'
    +MakeHtmlCacheInfo(MyOwnLife_aCacheData[idx].sId,
      MyOwnLife_aCacheData[idx].sType,MyOwnLife_aCacheData[idx].sName,'',IntToStr(MyOwnLife_aCacheData[idx].iStatus))
    +'</div>'+CRLF
    +'<div style="%StyleMyOwnCacheTitle2%">'
    +'%ColumnHeaderSize%: <img src="%IconCacheSize'+MyOwnLife_aCacheData[idx].sSize+'%" '
    +'title="'+MyOwnLife_aCacheData[idx].sSize+'" />&nbsp;&nbsp;'
    +'%ColumnHeaderDifficulty%: <img src="%IconCacheDiff'+MyOwnLife_aCacheData[idx].sDifficult+'%" '
    +'title="'+MyOwnLife_aCacheData[idx].sDifficult+'" />&nbsp;&nbsp;'
    +'%ColumnHeaderTerrain%: <img src="%IconCacheTerr'+MyOwnLife_aCacheData[idx].sTerrain+'%" '
    +'title="'+MyOwnLife_aCacheData[idx].sTerrain+'" />&nbsp;&nbsp;'
    +ReplaceString(ReplaceString(GlobalVarReplace('%ColumnHeadertag-Elevation%: '),'<br />',' '),'<br />',' ')
    +MyOwnLife_aCacheData[idx].sElevation
    +' m</div>';
  aPar.Values['Title']:=s;
  s:=ProcessCommand('CmdTitle',aPar);
  sOut:='<div id="'+MyOwnLife_GetIdElement(sMainItem,idx)+'" ';//ID divu pro skryti kese
  //pokud je kes archivovana, nastavime ji neviditelnou
  if(MyOwnLife_aCacheData[idx].bArchived) then sOut:=sOut+'style="display:none;"';
  sOut:=sOut+'>'+CRLF+s;
  Output(sOut);
  Output(MyOwnLife_TableLogs(idx));         //vytvorime prehled logu
  // graf pomo zapisuje do vstupu!
  Output('<table><tr><td>');
  if(not MyOwnLife_aCacheData[idx].bEvent) then
    MyOwnLife_GraphCache(idx, aPar)
  else
    MyOwnLife_GraphEvent(idx, aPar);
  Output('</td><td>');
  MyOwnLife_TableCache(idx,aPar);  //doplnime tabulku s informacemi
  Output('</td></tr></table>');
  if(not MyOwnLife_aCacheData[idx].bEvent) and (aPar.Values['MonthYear']='Yes')
    then MyOwnLife_MonthYear(idx, aPar);
  Output('</div>'+CRLF);
end;

//hlavni funkce modulu
function MyOwnLife_Run(aPar:TStringList):string;
var s,s1,sMainItem:string;
    i,n:integer;
begin
  Result := '';
  //Clear---------------------------------
  SetLength(MyOwnLife_aCacheData,0);
  MyOwnLife_Param.sCaches:='';
  MyOwnLife_Param.sCachesSQL:='';
  MyOwnLife_nCaches:=0;
  MyOwnLife_iMax:=0;
  //--------------------------------------
  //Log(aPar.CommaText);
  //Header
  if(aPar.Values['Title']<>'') then Result:=ProcessCommand('CmdTitle',aPar);
  //--------------------------------------
  MyOwnLife_Param.sTagXTF:=aPar.Values['TagXTF'];
  MyOwnLife_Param.bWordLogs:=(UpperCase(aPar.Values['WordLogs'])='YES');
  MyOwnLife_Param.bFP:=(UpperCase(aPar.Values['FavoritPoints'])='YES');
  //--------------------------------------
  //Zpracovn vlastnch ke z nastaven
  //--------------------------------------
  s:=aPar.Values['Caches'];
  //pokud je seznam kesi prazdny, nacte vsechny kese pomoci SQL (to je pro
  //pripad, ze by uzivatel zapomnel seznam vypolnit)
  if(s='') then begin
    Log(_('INFO: List of caches not set. Loading all ...'));
    //ziskame SQL prikaz z parametru (to aby byl urcite stejny)
    s:=GlobalVarget('OwnedIDsForInPhrase',false);
    s:=ReplaceString(ReplaceString(s,'''',''),'"',''); //zbavime se uvozovek/apostrofu
    s:=ReplaceString(s,',',CRLF);
  end
  else begin
    s:=ReplaceString(s,'|',CRLF);              //pokud jsou kese primo zadane, rozdelime je
  end;
  //v s je seznam ID kodu oddeleny CRLF
  //ShowMessage('Seznam vlastnich kesi:'+CRLF+CRLF+s);
  while(s<>'') do begin
    s1:=Fetch(s,CRLF);
    if(MyOwnLife_Param.sCaches<>'') then MyOwnLife_Param.sCaches:=MyOwnLife_Param.sCaches+',';
    MyOwnLife_Param.sCaches:=MyOwnLife_Param.sCaches+Fetch(s1,'-'); //jen GC kod bez jmena kese
  end;
  Log('INFO: caches for MyOwnLife='+MyOwnLife_Param.sCaches);
  MyOwnLife_Param.sCachesSQL:=''''+ReplaceString(MyOwnLife_Param.sCaches,',',''',''')+'''';
  //ShowMessage('Seznam vlastnich kesi:'+CRLF+CRLF+MyOwnLife_Param.sCaches+CRLF+CRLF+MyOwnLife_Param.sCachesSQL);
  MyOwnLife_nCaches:=MyOwnLife_PrepareData();
  if(MyOwnLife_Param.bWordLogs) then MyOwnLife_PrepareLogs();
  //--------------------------------------
  //--------------------------------------
  //zkontrolujeme, zda jsou vubec nejake vlastni kesky
  if(Length(MyOwnLife_aCacheData)=0) then begin
    //7.1.2019, aby nebylo preruseno zpracovani a chyby se vypsaly jen na konci
    //AddError(_('ERR: No owned caches found!'));
    Log(_('ERR: No owned caches found!'));
		exit;
  end;
  //vytvorime tlacitka pro zobrazeni/skryti archivivanych kesi
  sMainItem:='iMOLife'+IntToStr(iItem);
  //pokud neni zadna archivovana keska, tlacitka nedelame
  n:=0;
  for i:=Low(MyOwnLife_aCacheData) to High(MyOwnLife_aCacheData) do begin
    if(MyOwnLife_aCacheData[i].bArchived) then Inc(n);
  end;
  if(n>0) then Result:=Result+MyOwnLife_Buttons(sMainItem);
  Result:=EChartsTheme()+Result; //vlozime standardni temata, pokud je potreba
  Output(Result);
  Result:='';
  //radit kese od nejstarsi po nejmladsi
  for i:=Low(MyOwnLife_aCacheData) to High(MyOwnLife_aCacheData) do begin
    MyOwnLife_OutputCache(i,sMainItem,aPar);
  end;
  //Free memory
  SetLength(MyOwnLife_aCacheData,0);
end; 
